/**
 * op.C: This file contains code for the ChanServ OP functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: op.C,v 1.8 2007/09/22 14:27:27 pippijn Exp $
 */

#include "atheme.h"
#include <ermyth/module.h>
#include <account/chanacs.h>
#include <account/mychan.h>

static char const rcsid[] = "$Id: op.C,v 1.8 2007/09/22 14:27:27 pippijn Exp $";

REGISTER_MODULE ("chanserv/op", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void cs_cmd_op (sourceinfo_t *si, int parc, char *parv[]);
static void cs_cmd_deop (sourceinfo_t *si, int parc, char *parv[]);

command_t const cs_op = { "OP", N_("Gives channel ops to a user."), AC_NONE, 2, cs_cmd_op };
command_t const cs_deop = { "DEOP", N_("Removes channel ops from a user."), AC_NONE, 2, cs_cmd_deop };

E cmdvec cs_cmdtree;
E helpvec cs_helptree;

bool
_modinit (module *m)
{
  cs_cmdtree << cs_op;
  cs_cmdtree << cs_deop;

  help_addentry (cs_helptree, "OP", "help/chanserv/op", NULL);
  help_addentry (cs_helptree, "DEOP", "help/chanserv/deop", NULL);

  return true;
}

void
_moddeinit ()
{
  cs_cmdtree >> cs_op;
  cs_cmdtree >> cs_deop;

  help_delentry (cs_helptree, "OP");
  help_delentry (cs_helptree, "DEOP");
}

static void
cs_cmd_op (sourceinfo_t *si, int parc, char *parv[])
{
  char *chan = parv[0];
  char *nick = parv[1];
  mychan_t *mc;
  user_t *tu;
  chanuser_t *cu;

  if (!chan)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "OP");
      command_fail (si, fault::needmoreparams, _("Syntax: OP <#channel> [nickname]"));
      return;
    }

  mc = mychan_t::find (chan);
  if (!mc)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not registered."), chan);
      return;
    }

  if (!chanacs_source_has_flag (mc, si, CA_OP))
    {
      command_fail (si, fault::noprivs, _("You are not authorized to perform this operation."));
      return;
    }

  if (mc->find_metadata ("private:close:closer"))
    {
      command_fail (si, fault::noprivs, _("\2%s\2 is closed."), chan);
      return;
    }

  /* figure out who we're going to op */
  if (!nick)
    tu = si->su;
  else
    {
      if (!(tu = user_find_named (nick)))
	{
	  command_fail (si, fault::nosuch_target, _("\2%s\2 is not online."), nick);
	  return;
	}
    }

  if (is_internal_client (tu))
    return;

  /* SECURE check; we can skip this if sender == target, because we already verified */
  if ((si->su != tu) && (mc->flags & MC_SECURE) && !chanacs_user_has_flag (mc, tu, CA_OP) && !chanacs_user_has_flag (mc, tu, CA_AUTOOP))
    {
      command_fail (si, fault::noprivs, _("You are not authorized to perform this operation."), mc->name);
      command_fail (si, fault::noprivs, _("\2%s\2 has the SECURE option enabled, and \2%s\2 does not have appropriate access."), mc->name, tu->nick);
      return;
    }

  cu = chanuser_find (mc->chan, tu);
  if (!cu)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not on \2%s\2."), tu->nick, mc->name);
      return;
    }

  modestack_mode_param (chansvs.nick, mc->chan, MTYPE_ADD, 'o', CLIENT_NAME (tu));
  cu->modes |= CMODE_OP;

  if (si->c == NULL && tu != si->su)
    notice (chansvs.nick, tu->nick, "You have been opped on %s by %s", mc->name, get_source_name (si));

  logcommand (si, CMDLOG_SET, "%s OP %s!%s@%s", mc->name, tu->nick, tu->user, tu->vhost);
  if (!chanuser_find (mc->chan, si->su))
    command_success_nodata (si, _("\2%s\2 has been opped on \2%s\2."), tu->nick, mc->name);
}

static void
cs_cmd_deop (sourceinfo_t *si, int parc, char *parv[])
{
  char *chan = parv[0];
  char *nick = parv[1];
  mychan_t *mc;
  user_t *tu;
  chanuser_t *cu;

  if (!chan)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "DEOP");
      command_fail (si, fault::needmoreparams, _("Syntax: DEOP <#channel> [nickname]"));
      return;
    }

  mc = mychan_t::find (chan);
  if (!mc)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not registered."), chan);
      return;
    }

  if (!chanacs_source_has_flag (mc, si, CA_OP))
    {
      command_fail (si, fault::noprivs, _("You are not authorized to perform this operation."));
      return;
    }

  if (mc->find_metadata ("private:close:closer"))
    {
      command_fail (si, fault::noprivs, _("\2%s\2 is closed."), chan);
      return;
    }

  /* figure out who we're going to deop */
  if (!nick)
    tu = si->su;
  else
    {
      if (!(tu = user_find_named (nick)))
	{
	  command_fail (si, fault::nosuch_target, _("\2%s\2 is not online."), nick);
	  return;
	}
    }

  if (is_internal_client (tu))
    return;

  cu = chanuser_find (mc->chan, tu);
  if (!cu)
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not on \2%s\2."), tu->nick, mc->name);
      return;
    }

  modestack_mode_param (chansvs.nick, mc->chan, MTYPE_DEL, 'o', CLIENT_NAME (tu));
  cu->modes &= ~CMODE_OP;

  if (si->c == NULL && tu != si->su)
    notice (chansvs.nick, tu->nick, "You have been deopped on %s by %s", mc->name, get_source_name (si));

  logcommand (si, CMDLOG_SET, "%s DEOP %s!%s@%s", mc->name, tu->nick, tu->user, tu->vhost);
  if (!chanuser_find (mc->chan, si->su))
    command_success_nodata (si, _("\2%s\2 has been deopped on \2%s\2."), tu->nick, mc->name);
}

/* vim:cinoptions=>s,e0,n0,f0,{0,}0,^0,=s,ps,t0,c3,+s,(2s,us,)20,*30,gs,hs
 * vim:ts=8
 * vim:sw=8
 * vim:noexpandtab
 */
